/*
 * $Id: archive.c,v 1.2 1998/01/07 23:30:15 labovit Exp $
 */

/* NOTE
 * All of these routines should ONLY be called via the connection thread!
 * The connection thread owns this memory
 */

#include <sys/types.h> 
#include <sys/stat.h>
#include <fcntl.h>
#include <mrt.h>
#include <sys/time.h>
#include <select.h>
#include "salserver.h"

extern trace_t			*default_trace; 
extern salserver_t		SALSERVER; 

int sal_archive (data_pdu_t *data) {
  char tmp[512], date[512];
  int fd;
  sal_archive_t *archive;
  time_t t;
  struct tm tms;
  sal_buffer_t *buffer;
  int n;

  SALSERVER.where = "sal_archive";

  if (data->time <= 0) 
    t = time (NULL);
  else
    t = data->time;
#ifdef HAVE_LOCALTIME_R
  localtime_r (&t, &tms);
  strftime (date, 512, "%y%m%d", &tms);
#else
  strftime (date, 512, "%y%m%d", localtime (&t));
#endif /* HAVE_LOCALTIME_R */


  /* make directory */
  /* this could be REALLY, REALLY DANGEROUS -- overwrite system files
   * we need sanity/security checks here!
   */
  sprintf (tmp, "%s/%s", SALSERVER.dir, data->name);
  mkdir(tmp, S_IRWXU | S_IROTH | S_IXOTH);

  /* save to disk */
  sprintf (tmp, "%s/%s/%s.last", SALSERVER.dir, data->name, data->name);
  trace (TRACE, default_trace, "Saving to %s\n", tmp);
  
  if ((fd = open (tmp, O_WRONLY|O_CREAT, S_IRUSR|S_IWUSR|S_IROTH)) <= 0) {
    trace (TRACE, default_trace, 
	   "ERROR -- could not open for writing %s\n", tmp);
    return (-1);
  }

  buffer = New_Sal_Buffer (12 + data->proplen + data->datalen);

  UTIL_PUT_NETLONG (SALSERVER.key, buffer->cp); /* key */
  memcpy (buffer->cp, thisInterfaceVersion, 4); /* interface version */
  buffer->cp +=4;
  UTIL_PUT_NETLONG (data->proplen, buffer->cp); /* property length */
  
  /* properties */
  memcpy (buffer->cp, data->propbuf, data->proplen);
  buffer->cp += data->proplen;
  memcpy (buffer->cp, data->databuf, data->datalen);

  n = 0;
  n = write (fd, buffer->data, buffer->size);	

  if (n < buffer->size) {
    trace (NORM, default_trace, "Buffering...\n");
    exit (0);
  }
  close (fd);

  /*
   * ARCHIVE DATE | TIME: 
   */
  LL_Iterate (SALSERVER.ll_archives, archive) {
    if (!strncmp (data->name, archive->name, strlen (archive->name))) { 
      
      /* TIME  */
      if (archive->flag == 1) {
	sprintf (tmp, "%s/%s/%s.time.%s", SALSERVER.dir, data->name, data->name, date);
	if ((fd = open (tmp, O_WRONLY|O_APPEND|O_CREAT, S_IRUSR|S_IWUSR|S_IROTH)) <= 0){
	  trace (TRACE, default_trace, 
		 "ERROR -- could not open for writing %s\n", tmp);
	  return (-1);
	}
      }
      /* DATE */
      else {
	sprintf (tmp, "%s/%s/%s.%s", SALSERVER.dir, data->name, data->name, date);
	if ((fd = open (tmp, O_WRONLY|O_CREAT, S_IRUSR|S_IWUSR|S_IROTH)) <= 0){
	  trace (TRACE, default_trace, 
		 "ERROR -- could not open for writing %s\n", tmp);
	  return (-1);
	}
      }
      trace (TR_TRACE, default_trace, "Archiving %s\n", tmp);
      write (fd, buffer->data, buffer->size);
      close (fd);
    }
  }

  Delete_Sal_Buffer (buffer);

  return (1);
}



/*
 * sal_ad_archive
 */
static int _sal_add_archive (char *channel, int type) {
  sal_archive_t *archive;
  
  archive = New (sal_archive_t);
  archive->flag = type;
  archive->name = strdup (channel);
  
  LL_Add (SALSERVER.ll_archives, archive);

  return (1);
}

int sal_add_archive (char *channel, int type) {
  schedule_event2 ("add_archive", SALSERVER.connection_schedule, 
		   (void *) _sal_add_archive, 2, 
		   channel, type);
  
  return (1);
}




void get_archive_config (void) {
  char *typestr[] = {"DATE", "TIME"};
  sal_archive_t *archive;

  LL_Iterate (SALSERVER.ll_archives, archive) {
    config_add_output ("archive %s %s\r\n", archive->name, typestr[archive->flag]);
  }
  signal_config_data_ready ();
  return;
}

