/*
 * $Id: net.c,v 1.2 1998/03/18 16:18:57 labovit Exp $
 */

#include <sys/types.h> 
#include <fcntl.h>
#include <mrt.h>
#include <select.h> 
#include <netinet/in.h>
#include <arpa/inet.h>
#include "salserver.h"

extern trace_t			*default_trace;
extern salserver_t		SALSERVER;



/* udp_recv
 * just for backward compatability -- return port 9000 to anyone who sends a request in 
 * on this udp port 
 */
int udp_recv () {
  char buf[512];
  struct sockaddr_in from;
  int fromlen;
  char *cp = buf;

  /* We have something! */
  fromlen = sizeof (from);
  if (recvfrom(SALSERVER.udp_sockfd, buf, sizeof(buf), 0, 
	       (struct sockaddr *) &from, &fromlen) < 0) {
    trace (NORM, default_trace, "Received malformed UDP packet from %s\n",
	   inet_ntoa (from.sin_addr));
    select_enable_fd (SALSERVER.udp_sockfd);
    return -1;
  }
  trace (TR_PACKET, default_trace, 
	 "Received UDP packet from %s \n", inet_ntoa (from.sin_addr));
  select_enable_fd (SALSERVER.udp_sockfd);

  /* send back UDP response */
  memset (buf, 0, 512);
  UTIL_PUT_NETSHORT (9000, cp); /* port */
  UTIL_PUT_NETSHORT (0, cp); /* emtpy */
  UTIL_PUT_NETLONG (123456, cp); /* key */

  sendto(SALSERVER.udp_sockfd, buf, 8, 0, (struct sockaddr *) &from, fromlen);
  trace (TR_PACKET, default_trace, "Sending UDP port 9000 reply to %s\n", 
	 inet_ntoa (from.sin_addr));
  return (1);
}



/* listen_udp_port
 * backward compatability -- listen on udp port 8086...
 */
int listen_udp_port () {
  struct sockaddr_in serv_addr;
  int optval = 1;
  
  serv_addr.sin_port = htons(SALSERVER.udp_port);
  serv_addr.sin_family = AF_INET;
  serv_addr.sin_addr.s_addr = htonl (INADDR_ANY);

  if ( (SALSERVER.udp_sockfd = socket (AF_INET, SOCK_DGRAM, 0)) < 0) {
    trace (NORM, default_trace, "ERROR -- Could not get socket (%s)\n",
	   strerror (errno));
    return (-1);
  }

  setsockopt (SALSERVER.udp_sockfd, SOL_SOCKET, SO_REUSEADDR,  
	      (char *) &optval, sizeof (optval));

  if (bind (SALSERVER.udp_sockfd, (struct sockaddr *) &serv_addr, 
	    sizeof (serv_addr)) < 0) {
      trace (NORM, default_trace, "ERROR -- BIND Failed in listen_udp_port (%s)\n",
	     strerror (errno));
      return (-1);
  }

  trace (NORM, default_trace, 
	"Listening for client udp request on port %d (socket %d)\n", 
	 SALSERVER.udp_port, SALSERVER.udp_sockfd);

  select_add_fd (SALSERVER.udp_sockfd, 1, (void *) udp_recv, NULL);

  return (1);


}







/* listen_udp_port
 * backward compatability -- listen on udp port 8086...
 */
int listen_tcp_port () {
  struct sockaddr_in serv_addr;
  int optval = 1;

  serv_addr.sin_port = htons(SALSERVER.tcp_port);
  serv_addr.sin_family = AF_INET;
  serv_addr.sin_addr.s_addr = htonl (INADDR_ANY);

  if ( (SALSERVER.tcp_sockfd = socket (AF_INET, SOCK_STREAM, 0)) < 0) {
    trace (NORM, default_trace, "ERROR -- Could not get socket (%s)\n",
	   strerror (errno));
    return (-1);
  }

  setsockopt (SALSERVER.tcp_sockfd, SOL_SOCKET, SO_REUSEADDR,  
	      (char *) &optval, sizeof (optval));


  if (bind (SALSERVER.tcp_sockfd, (struct sockaddr *) &serv_addr, 
	    sizeof (serv_addr)) < 0) {
      trace (NORM, default_trace, "ERROR -- BIND Failed in listen_tcp_port (%s)\n",
	     strerror (errno));
      return (-1);
  }

  listen (SALSERVER.tcp_sockfd, 500);

  trace (NORM, default_trace, 
	"Listening for tcp connections port %d (socket %d)\n", 
	 SALSERVER.tcp_port, SALSERVER.tcp_sockfd);

  select_add_fd (SALSERVER.tcp_sockfd, 1, (void *) schedule_connection_tcp_recv, NULL);
  select_enable_fd (SALSERVER.tcp_sockfd);

  return (1);


}

